@@ -3,25 +3,27 @@ High-level integration tests of diff-cover tool.
 """
 from __future__ import unicode_literals
 
+import io
 import os
 import os.path
 import re
 import shutil
 import tempfile
+import unittest
 from collections import defaultdict
 from io import BytesIO
 from subprocess import Popen
-import unittest
-import io
-import six
 
+import pylint
+import six
 from diff_cover.command_runner import CommandError
+from diff_cover.diff_cover_tool import main as diff_cover_main
+from diff_cover.diff_quality_tool import QUALITY_DRIVERS
+from diff_cover.diff_quality_tool import main as diff_quality_main
 from diff_cover.git_path import GitPathTool
-from diff_cover.tests.helpers import fixture_path, \
-    assert_long_str_equal
-from diff_cover.tool import main, QUALITY_DRIVERS
+from diff_cover.tests.helpers import assert_long_str_equal, fixture_path
 from diff_cover.violationsreporters.base import QualityDriver
-from mock import patch, Mock
+from mock import Mock, patch
 
 
 class ToolsIntegrationBase(unittest.TestCase):
@@ -41,11 +43,11 @@ class ToolsIntegrationBase(unittest.TestCase):
         cwd = os.getcwd()
 
         self._mock_popen = patch('subprocess.Popen').start()
-        self._mock_sys = patch('diff_cover.tool.sys').start()
+        self._mock_sys = patch('{}.sys'.format(self.tool_module)).start()
         try:
-            self._mock_getcwd = patch('diff_cover.tool.os.getcwdu').start()
+            self._mock_getcwd = patch('{}.os.getcwdu'.format(self.tool_module)).start()
         except AttributeError:
-            self._mock_getcwd = patch('diff_cover.tool.os.getcwd').start()
+            self._mock_getcwd = patch('{}.os.getcwd'.format(self.tool_module)).start()
         self._git_root_path = cwd
         self._mock_getcwd.return_value = self._git_root_path
 
@@ -99,8 +101,12 @@ class ToolsIntegrationBase(unittest.TestCase):
             args += ['--external-css-file', css_file]
 
         # Execute the tool
-        code = main(args)
-        self.assertEquals(code, expected_status)
+        if 'diff-cover' in args[0]:
+            code = diff_cover_main(args)
+        else:
+            code = diff_quality_main(args)
+
+        self.assertEqual(code, expected_status)
 
         # Check the HTML report
         with io.open(expected_html_path, encoding='utf-8') as expected_file:
@@ -138,9 +144,12 @@ class ToolsIntegrationBase(unittest.TestCase):
         self._capture_stdout(string_buffer)
 
         # Execute the tool
-        code = main(tool_args)
+        if 'diff-cover' in tool_args[0]:
+            code = diff_cover_main(tool_args)
+        else:
+            code = diff_quality_main(tool_args)
 
-        self.assertEquals(code, expected_status)
+        self.assertEqual(code, expected_status)
 
         # Check the console report
         with open(expected_console_path) as expected_file:
@@ -166,7 +175,8 @@ class ToolsIntegrationBase(unittest.TestCase):
         a phony directory.
         """
         def patch_diff(command, **kwargs):
-            if command[0:4] == ['git', '-c', 'diff.mnemonicprefix=no', 'diff']:
+            if command[0:6] == ['git', '-c', 'diff.mnemonicprefix=no', '-c',
+                                'diff.noprefix=no', 'diff']:
                 mock = Mock()
                 mock.communicate.return_value = (stdout, stderr)
                 mock.returncode = returncode
@@ -188,6 +198,8 @@ class DiffCoverIntegrationTest(ToolsIntegrationBase):
     The `git diff` is a mock, but everything else is our code.
     """
 
+    tool_module = 'diff_cover.diff_cover_tool'
+
     def test_added_file_html(self):
         self._check_html_report(
             'git_diff_add.txt',
@@ -369,13 +381,14 @@ class DiffCoverIntegrationTest(ToolsIntegrationBase):
 
         # Expect an error
         with self.assertRaises(CommandError):
-            main(['diff-cover', 'coverage.xml'])
+            diff_cover_main(['diff-cover', 'coverage.xml'])
 
 
 class DiffQualityIntegrationTest(ToolsIntegrationBase):
     """
     High-level integration test.
     """
+    tool_module = 'diff_cover.diff_quality_tool'
 
     def test_git_diff_error_diff_quality(self):
 
@@ -384,7 +397,7 @@ class DiffQualityIntegrationTest(ToolsIntegrationBase):
 
         # Expect an error
         with self.assertRaises(CommandError):
-            main(['diff-quality', '--violations', 'pycodestyle'])
+            diff_quality_main(['diff-quality', '--violations', 'pycodestyle'])
 
     def test_added_file_pycodestyle_html(self):
         self._check_html_report(
@@ -479,9 +492,14 @@ class DiffQualityIntegrationTest(ToolsIntegrationBase):
         )
 
     def test_added_file_pylint_console(self):
+        # output text was modified in https://github.com/PyCQA/pylint/pull/2036
+        if tuple(map(int, pylint.__version__.split('.'))) < (2, 4, 0):
+            console_report = 'pylint_violations_console_report.txt'
+        else:
+            console_report = 'pylint_violations_console_report_pylint_2.4.0.txt'
         self._check_console_report(
             'git_diff_violations.txt',
-            'pylint_violations_console_report.txt',
+            console_report,
             ['diff-quality', '--violations=pylint'],
         )
 
@@ -532,11 +550,11 @@ class DiffQualityIntegrationTest(ToolsIntegrationBase):
         with io.open('git_diff_add.txt', encoding='utf-8') as git_diff_file:
             self._set_git_diff_output(git_diff_file.read(), "")
         argv = ['diff-quality',
-                '--violations={0}'.format(tool_name),
+                '--violations={}'.format(tool_name),
                 report_arg]
 
-        with patch('diff_cover.tool.LOGGER') as logger:
-            exit_value = main(argv)
+        with patch('diff_cover.diff_quality_tool.LOGGER') as logger:
+            exit_value = diff_quality_main(argv)
             logger.error.assert_called_with(expected_error)
             self.assertEqual(exit_value, 1)
 
@@ -583,4 +601,3 @@ class DoNothingDriver(QualityDriver):
 
     def installed(self):
         return False
-
