@@ -16,12 +16,12 @@ from diff_cover.violationsreporters.base import BaseViolationReporter, Violation
 
 class XmlCoverageReporter(BaseViolationReporter):
     """
-    Query information from a Cobertura XML coverage report.
+    Query information from a Cobertura|Clover|JaCoCo XML coverage report.
     """
 
-    def __init__(self, xml_roots):
+    def __init__(self, xml_roots, src_roots=['']):
         """
-        Load the Cobertura XML coverage report represented
+        Load the XML coverage report represented
         by the cElementTree with root element `xml_root`.
         """
         super(XmlCoverageReporter, self).__init__("XML")
@@ -31,6 +31,8 @@ class XmlCoverageReporter(BaseViolationReporter):
         # Keys are source file paths, values are output of `violations()`
         self._info_cache = defaultdict(list)
 
+        self._src_roots = src_roots
+
     @staticmethod
     def _to_unix_path(path):
         """
@@ -44,7 +46,7 @@ class XmlCoverageReporter(BaseViolationReporter):
         :param path: string of the path to convert
         :return: the unix version of that path
         """
-        return posixpath.normpath(path.replace("\\", '/'))
+        return posixpath.normpath(os.path.normcase(path).replace("\\", '/'))
 
     def _get_classes(self, xml_document, src_path):
         """
@@ -74,7 +76,7 @@ class XmlCoverageReporter(BaseViolationReporter):
         # cobertura sometimes provides the sources for the measurements
         # within it. If we have that we outta use it
         sources = xml_document.findall('sources/source')
-        sources = [source.text for source in sources]
+        sources = [source.text for source in sources if source.text]
         classes = [class_tree
                    for class_tree in xml_document.findall(".//class")
                    or []]
@@ -84,7 +86,7 @@ class XmlCoverageReporter(BaseViolationReporter):
              src_abs_path in [
                  self._to_unix_path(
                      os.path.join(
-                         source,
+                         source.strip(),
                          clazz.get('filename')
                      )
                  ) for source in sources]]
@@ -97,14 +99,7 @@ class XmlCoverageReporter(BaseViolationReporter):
         )
         return classes
 
-    def _get_src_path_line_nodes(self, xml_document, src_path):
-        """
-        Returns a list of nodes containing line information for `src_path`
-        in `xml_document`.
-
-        If file is not present in `xml_document`, return None
-        """
-
+    def _get_src_path_line_nodes_cobertura(self, xml_document, src_path):
         classes = self._get_classes(xml_document, src_path)
 
         if not classes:
@@ -113,6 +108,58 @@ class XmlCoverageReporter(BaseViolationReporter):
             lines = [clazz.findall('./lines/line') for clazz in classes]
             return [elem for elem in itertools.chain(*lines)]
 
+    def _get_src_path_line_nodes_clover(self, xml_document, src_path):
+        """
+        Return a list of nodes containing line information for `src_path`
+        in `xml_document`.
+
+        If file is not present in `xml_document`, return None
+        """
+
+        files = [file_tree
+                 for file_tree in xml_document.findall(".//file")
+                 if GitPathTool.relative_path(file_tree.get('path')) == src_path
+                 or []]
+        if not files:
+            return None
+        lines = [file_tree.findall('./line[@type="stmt"]')
+                 for file_tree in files]
+        return [elem for elem in itertools.chain(*lines)]
+
+    def _measured_source_path_matches(self, package_name, file_name, src_path):
+        # find src_path in any of the source roots
+        if not src_path.endswith(file_name):
+            return None
+
+        norm_src_path = os.path.normcase(src_path);
+        for root in self._src_roots:
+            if os.path.normcase(GitPathTool.relative_path(os.path.join(root, package_name, file_name))) == norm_src_path:
+                return True
+
+    def _get_src_path_line_nodes_jacoco(self, xml_document, src_path):
+        """
+        Return a list of nodes containing line information for `src_path`
+        in `xml_document`.
+
+        If file is not present in `xml_document`, return None
+        """
+
+        files = []
+        packages = [pkg for pkg in xml_document.findall(".//package")]
+        for pkg in packages:
+            _files = [_file
+                      for _file in pkg.findall('sourcefile')
+                      if self._measured_source_path_matches(pkg.get('name'), _file.get('name'), src_path)
+                      or []
+                      ]
+            files.extend(_files)
+
+        if not files:
+            return None
+        lines = [file_tree.findall('./line')
+                 for file_tree in files]
+        return [elem for elem in itertools.chain(*lines)]
+
     def _cache_file(self, src_path):
         """
         Load the data from `self._xml_roots`
@@ -132,33 +179,45 @@ class XmlCoverageReporter(BaseViolationReporter):
 
             # Loop through the files that contain the xml roots
             for xml_document in self._xml_roots:
-                line_nodes = self._get_src_path_line_nodes(xml_document,
-                                                           src_path)
-
+                if xml_document.findall('.[@clover]'):
+                    # see etc/schema/clover.xsd at  https://bitbucket.org/atlassian/clover/src
+                    line_nodes = self._get_src_path_line_nodes_clover(xml_document, src_path)
+                    _number = 'num'
+                    _hits = 'count'
+                elif xml_document.findall('.[@name]'):
+                    # https://github.com/jacoco/jacoco/blob/master/org.jacoco.report/src/org/jacoco/report/xml/report.dtd
+                    line_nodes = self._get_src_path_line_nodes_jacoco(xml_document, src_path)
+                    _number = 'nr'
+                    _hits = 'ci'
+                else:
+                    # https://github.com/cobertura/web/blob/master/htdocs/xml/coverage-04.dtd
+                    line_nodes = self._get_src_path_line_nodes_cobertura(xml_document, src_path)
+                    _number = 'number'
+                    _hits = 'hits'
                 if line_nodes is None:
                     continue
 
                 # First case, need to define violations initially
                 if violations is None:
-                    violations = set(
-                        Violation(int(line.get('number')), None)
+                    violations = {
+                        Violation(int(line.get(_number)), None)
                         for line in line_nodes
-                        if int(line.get('hits', 0)) == 0)
+                        if int(line.get(_hits, 0)) == 0}
 
                 # If we already have a violations set,
                 # take the intersection of the new
                 # violations set and its old self
                 else:
-                    violations = violations & set(
-                        Violation(int(line.get('number')), None)
+                    violations = violations & {
+                        Violation(int(line.get(_number)), None)
                         for line in line_nodes
-                        if int(line.get('hits', 0)) == 0
-                    )
+                        if int(line.get(_hits, 0)) == 0
+                    }
 
                 # Measured is the union of itself and the new measured
-                measured = measured | set(
-                    int(line.get('number')) for line in line_nodes
-                )
+                measured = measured | {
+                    int(line.get(_number)) for line in line_nodes
+                }
 
             # If we don't have any information about the source file,
             # don't report any violations
@@ -361,9 +420,9 @@ class PylintDriver(QualityDriver):
                         # If we're looking for a particular source file,
                         # ignore any other source files.
                         if function_name:
-                            error_str = u"{0}: {1}: {2}".format(pylint_code, function_name, message)
+                            error_str = "{}: {}: {}".format(pylint_code, function_name, message)
                         else:
-                            error_str = u"{0}: {1}".format(pylint_code, message)
+                            error_str = "{}: {}".format(pylint_code, message)
 
                         violation = Violation(int(line_number), error_str)
                         violations_dict[pylint_src_path].append(violation)
@@ -376,3 +435,61 @@ class PylintDriver(QualityDriver):
         Returns: boolean True if installed
         """
         return run_command_for_code(self.command_to_check_install) == 0
+
+class CppcheckDriver(QualityDriver):
+    """
+    Driver for cppcheck c/c++ static analyzer.
+    """
+    def __init__(self):
+        """
+        args:
+            expression: regex used to parse report
+        See super for other args
+        """
+        super(CppcheckDriver, self).__init__(
+            'cppcheck',
+            ['c', 'cpp', 'h', 'hpp'],
+            ['cppcheck', '--quiet'],
+            output_stderr=True,
+        )
+        # Errors look like:
+        # [src/foo.c:123]: (error) Array 'yolo[4]' accessed at index 4, which is out of bounds.
+        # Match for everything, including ":" in the file name (first capturing
+        # group), in case there are pathological path names with ":"
+        self.cppcheck_expression = re.compile(r"^\[(.*?):(\d+)\]: (.*$)")
+        self.command_to_check_install = ['cppcheck', '--version']
+
+    def parse_reports(self, reports):
+        """
+        Args:
+            reports: list[str] - output from the report
+        Return:
+            A dict[Str:Violation]
+            Violation is a simple named tuple Defined above
+        """
+        violations_dict = defaultdict(list)
+        for report in reports:
+            output_lines = report.splitlines()
+
+            for line in output_lines:
+                match = self.cppcheck_expression.match(line)
+
+                # Ignore any line that isn't matched
+                # (for example, snippets from the source code)
+                if match is not None:
+
+                    (cppcheck_src_path,
+                     line_number,
+                     message) = match.groups()
+
+                    violation = Violation(int(line_number), message)
+                    violations_dict[cppcheck_src_path].append(violation)
+
+        return violations_dict
+
+    def installed(self):
+        """
+        Method checks if the provided tool is installed.
+        Returns: boolean True if installed
+        """
+        return run_command_for_code(self.command_to_check_install) == 0
